class PSingleton:
    def __init__(self, decorated):
        self._decorated = decorated

    def GetInstance(self):
        try:
            return self._instance
        except AttributeError:
            self._instance = self._decorated()
            return self._instance

    def __call__(self):
        raise TypeError('Singletons must be accessed through `GetInstance()`.')

    def __instancecheck__(self, inst):
        return isinstance(inst, self._decorated)

_DEBUG, _WARNING, _ERROR = 0xB1, 0xB2, 0xB3
_LVSTR = {
    _DEBUG   : "  DEBUG",
    _WARNING : "WARNING",
    _ERROR   : "  ERROR"
}

@PSingleton
class PLogger:
    def __init__(self):
        self.level = _ERROR
        self.initialized = False

    def Init(self, level):
        self.level = level
        self.initialized = True

    def Log(self, level, format, argv):
        if level < self.level:
            return
        print "[" + _LVSTR[level] + "]", format % argv

def INIT(level=_ERROR):
    PLogger.GetInstance().Init(level)

def DEBUG(format, *argv):
    PLogger.GetInstance().Log(_DEBUG, format, argv)

def WARNING(format, *argv):
    PLogger.GetInstance().Log(_WARNING, format, argv)

def ERROR(format, *argv):
    PLogger.GetInstance().Log(_ERROR, format, argv)

'''
INIT(level = _DEBUG)
DEBUG("debug : %d, %s", 1, "aa")
WARNING("warning : %d, %s", 1, "aa")
ERROR("error : %d, %s", 1, "aa")
'''
