from Plugins.Plugin import PluginDescriptor

from Components.Label import Label
from Components.config import config
from Components.ServiceEventTracker import ServiceEventTracker

from Screens.Screen import Screen
from Screens.ChoiceBox import ChoiceBox
from Screens.MessageBox import MessageBox
from Screens.InfoBar import InfoBar, MoviePlayer
from Screens.InfoBarGenerics import InfoBarNotifications

from enigma import eTimer, fbClass, eRCInput, eServiceReference, iPlayableService
from pipc import PServerThread, PServerHandlers

import plogger, pipc
import os, signal, struct

from enigma import getDesktop

pipc._SOCKETFILE = "/tmp/xbmc.support"
plogger.INIT(plogger._ERROR)

def dvbapp_lock():
    plogger.DEBUG("dvbapp_lock")
    fbClass.getInstance().lock()
    eRCInput.getInstance().lock()

def dvbapp_unlock():
    plogger.DEBUG("dvbapp_unlock")
    fbClass.getInstance().unlock()
    eRCInput.getInstance().unlock()

_g_launcher_handler = None

_g_dw,_g_dh = 1280,720
def SaveDesktopInfo():
        global _g_dw, _g_dh
        try:
                _g_dw = getDesktop(0).size().width()
                _g_dh = getDesktop(0).size().height()
        except: _g_dw,_g_dh = 1280,720
        print "[XBMC] Desktop size [%dx%d]" % (_g_dw,_g_dh)
	open("/tmp/dw.info", "w").write(str(_g_dw) + "x" + str(_g_dh))
SaveDesktopInfo()

class XBMCMoviePlayer(MoviePlayer, InfoBarNotifications):
    skin = """
        <screen name="KodiMoviePlayer" flags="wfNoBorder" position="%d,%d" size="1280,110" title="KodiMoviePlayer" backgroundColor="#41000000">
            <ePixmap position="302,33" size="122,72" pixmap="/usr/lib/enigma2/python/Plugins/Extensions/XBMC/button.png" zPosition="-1" transparent="1" alphatest="blend" />
            <widget source="session.CurrentService" render="PositionGauge" position="386,35" size="400,20" zPosition="2" pointer="skin_default/position_pointer.png:540,0" transparent="1" foregroundColor="#20224f">
                <convert type="ServicePosition">Gauge</convert>
            </widget>
            <widget source="session.CurrentService" render="Label" position="846,35" size="50,20" font="Regular;22" halign="center" valign="center" foregroundColor="#ffffff" backgroundColor="#41000000">
                <convert type="ServicePosition">Position</convert>
            </widget>
            <widget name="sidebar" position="916,35" size="10,20" font="Regular;20" halign="center" valign="center" foregroundColor="#ffffff" backgroundColor="#41000000" />
            <widget source="session.CurrentService" render="Label" position="916,35" size="50,20" font="Regular;22" halign="center" valign="center" foregroundColor="#ffffff" backgroundColor="#41000000"> 
                <convert type="ServicePosition">Length</convert>
            </widget>
        </screen>
        """ % ((_g_dw-1280)/2, _g_dh-110)
    #print "[XBMC] skin... >> %s" % (skin)
    def __init__(self, session, url):
        self.sref = eServiceReference(0x1001, 0, url)

        MoviePlayer.__init__(self, session, self.sref)
        InfoBarNotifications.__init__(self)

        session.nav.getCurrentService().streamed().setBufferSize(8*1024*1024)

        self.__event_tracker = ServiceEventTracker(screen=self, eventmap={
            iPlayableService.evBuffering: self.__evUpdatedBufferInfo,
        })
        self["sidebar"] = Label(_("/"))
    
    def __evUpdatedBufferInfo(self):
        bufferInfo = self.session.nav.getCurrentService().streamed().getBufferCharge()
        if bufferInfo[2] != 0:
            self.bufferSeconds = bufferInfo[4] / bufferInfo[2] #buffer size / avgOutRate
        else:
            self.bufferSeconds = 0
        self.bufferPercent = bufferInfo[0]
        self.bufferSecondsLeft = self.bufferSeconds * self.bufferPercent / 100
        if(self.bufferPercent > 90):
            self.bufferFull()
        #if(self.bufferPercent < 1):
        #    self.bufferEmpty()

    def bufferFull(self):
        if self.seekstate != self.SEEK_STATE_PLAY :
            self.setSeekState(self.SEEK_STATE_PLAY)
    
    def bufferEmpty(self):
        if self.seekstate != self.SEEK_STATE_PAUSE :
            self.setSeekState(self.SEEK_STATE_PAUSE)

    def leavePlayer(self):
        self.leavePlayerConfirmed(True)
 
    def leavePlayerConfirmed(self, answer):
        if answer:
            self.close()

    def doEofInternal(self, playing):
        self.leavePlayer()

    def isPlaying(self):
        return self.seekstate == self.SEEK_STATE_PLAY and True or False

_OPCODES = [ 'CONTROL_EXIT', 'PLAY_URL', 'PLAY_STATUS', 'PLAY_STOPED', 'OPCODE_END' ]
class XBMCPlayerHandlers(PServerHandlers):
    def __init__(self):
        PServerHandlers.__init__(self, _OPCODES, '_CBH_')
        self._cbPlayerExit()

    def _cbPlayerExit(self, ret=None):
        try:
            self.playerHandle.playlist.clear()
        except: pass
        self.playerHandle = None

    def _CBH_PLAY_STATUS(self, result, packet):
        if self.playerHandle is None:
            return (False, None)
        return (True, None)

    def _CBH_PLAY_URL(self, result, packet):
        global _g_launcher_handler
        if len(packet) == 0 or _g_launcher_handler is None:
            return (False, None)
        
        dvbapp_unlock()

        if self.playerHandle is not None:
            self.playerHandle.close()
            self.playerHandle = None

        session, url = _g_launcher_handler.session, packet
        plogger.DEBUG("[%s]" % (url))

        try :
            from Plugins.Extensions.MediaPlayer.plugin import MediaPlayer
            self.playerHandle = session.openWithCallback(self._cbPlayerExit, MediaPlayer)
            self.playerHandle.savePlaylistOnExit = False

            self.playerHandle.playlist.clear()
            self.playerHandle.playlist.addFile(eServiceReference(4097, 0, url))
            self.playerHandle.changeEntry(0)
            self.playerHandle.switchToPlayList()
        except:
            self.playerHandle = session.openWithCallback(self._cbPlayerExit, XBMCMoviePlayer, url)

        return (True, None)

    def _CBH_PLAY_STOPED(self, result, packet):
        dvbapp_lock()
        return (True, None)

    def _CBH_CONTROL_EXIT(self, result, packet):
        try:
            global _g_launcher_handler
            if _g_launcher_handler is not None:
                _g_launcher_handler.Exit()
                _g_launcher_handler = None
        except Exception, err:
            plogger.ERROR("%s", err)
            return (False, None)
        return (True, None)

_HANDLER = XBMCPlayerHandlers()
class XBMCLauncher(Screen):
    global _g_dw, _g_dh
    skin = """<screen name="KodiLauncher" position="0,0" size="%d,%d" backgroundColor="#41000000" flags="wfNoBorder" title=" "></screen>""" % (_g_dw, _g_dh)
    def __init__(self, session):
        self.session = session
        
        Screen.__init__(self, session)
        
        self.pServerThread = PServerThread()
        self.pServerThread.open(timeout = 1)
        self.pServerThread.start()
        
        self.closeTimer = eTimer()
        self.closeTimer.timeout.get().append(self._cbCloseTimer)
        
        global _g_launcher_handler
        _g_launcher_handler = self

        os.system("sh /usr/lib/kodi/kodi.run")

        self.beforeServiceRef = self.session.nav.getCurrentlyPlayingServiceReference()
        self.session.nav.stopService()

        self.dvbappLockTimer = eTimer()
        self.dvbappLockTimer.timeout.get().append(self._cbDvbappLockTimer)
        self.dvbappLockTimer.start(500)

        self.checkXBMCTimer = eTimer()
        self.checkXBMCTimer.timeout.get().append(self._cbCheckXBMCTimer)
        self.checkXBMCTimer.start(3000)

    def _cbDvbappLockTimer(self):
        self.dvbappLockTimer.stop()
        dvbapp_lock()

    def Exit(self):
        self.closeTimer.start(1500)
    
    def _cbCloseTimer(self):
        self.closeTimer.stop()

        self.pServerThread.kill()
        self.pServerThread.join()
        
        dvbapp_unlock()
        port = config.av.videoport.value
        mode = config.av.videomode[port].value
        rate = config.av.videorate[config.av.videomode[config.av.videoport.value].value].value
        rate = rate.replace('Hz','')
        cmd = 'echo "%s%s" > /proc/stb/video/videomode' % (mode,rate)
        os.system (cmd)

        if self.beforeServiceRef is not None:
                self.session.nav.playService(self.beforeServiceRef)
        self.close()

    def _cbCheckXBMCTimer(self):
        data = os.popen('ps -ef | grep kodi.bin | grep -v grep').read().strip()
	if data is None or data == "":
	    self.checkXBMCTimer.stop()
            os.system('xbmc.helper --stop')

def session_start_main(session, reason, **kwargs):
    PServerThread.close()

def plugin_start_xbmc(session, **kwargs):
    os.system ('echo "720p50" > /proc/stb/video/videomode')
    dvbapp_unlock()
    session.open(XBMCLauncher)

def Plugins(**kwargs):
    l = []
    l.append(PluginDescriptor(where=PluginDescriptor.WHERE_SESSIONSTART, fnc=session_start_main))
    l.append(PluginDescriptor(name=_("Kodi Launcher"), description=_("start kodi(formerly known as XBMC) mediacenter"), where=PluginDescriptor.WHERE_PLUGINMENU, fnc=plugin_start_xbmc))
    return l

